/************************************************************************/
/* 
<HTML>
<HEADER>
<TITLE>HTML Extract</TITLE>
</HEADER>
<BODY>
<H1> Module:   htmlextract (htmlextract.c) </H1><UL>
<LI> Purpose:  to extract HTML comments from C (or any) code.
<LI> Language/Compiler/O/S:   ANSI C, SCO CC or GNU gcc:  UNIX 
                              BCC 3.0:  DOS/Windows 3.1 
<LI> Author:      Wade Hampton
<LI> Date:        6/29/95
<LI> Revision History [from RCS]
<UL><PRE>

 $Log: htmlextract.c,v $
 * Revision 1.4  1995/09/26  15:43:43  wadeh
 * copyright and GPL added
 *
 * Revision 1.3  1995/09/26  15:12:35  wadeh
 * changed so it returns 0 when done (for make files)
 *
 * Revision 1.2  1995/09/26  14:58:48  wadeh
 * added command line args -H, -h, -?, -V.
 * added input file option -ifilename
 * added date and time in comments
 * added PrintHelp and PrintVersion functions
 *
 * Revision 1.1  1995/06/29  15:12:45  wadeh
 * Initial revision
 *

</PRE></UL>
</UL><H1> Notes: </H1><OL>
<LI> Usage:  htmlextract &lt;ifile.[c,C,CC,h] &gt;ifile.html
<LI>         htmlextract -iinfile &gt;ifile.html
<LI> Extracts text bracketed by <HTML> and </HTML>
<LI> Puts <HTML> at start, </HTML> at end
<LI> Uses stdin for input, stdout for output, or uses a file name.
<LI> To build:  [cc,gcc] [-g] -o htmlextract htmlextract.c
</OL>
</OL><H2>Requirements: </H2> <OL> 
<LI> (1) Extract HTML from source code for on-line documentation.
</OL> 

</HTML> 
 *----------------------------------------------------------------------*/
/* TYPES - - - - - - - - - - - - - - - - - - - - - -  - - - - - - - - - */
/* INCLUDE FILES - - - - - - - - - - - - - - - - - -  - - - - - - - - - */
#include <stdio.h>
#include <string.h>
#include "htmldef.h"

#include <sys/types.h>
#include <time.h>

/* CONSTANTS- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - *
<HTML>
<P>RCS ID String:  $Id: htmlextract.c,v 1.4 1995/09/26 15:43:43 wadeh Exp $<P>
</HTML> 
*/
static char rcsid[] = "$Id: htmlextract.c,v 1.4 1995/09/26 15:43:43 wadeh Exp $";   

#define TRUE (1==1)
#define FALSE (1==0)
/* #define INFILE    stdin */
#define OUTFILE   stdout
#define PROG_NAME "htmlextract"

/* LOCAL DATA - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

FILE *INFILE = stdin;  /* input file -- typ. stdin */

/* EXTERNAL PROCEDURES- - - - - - - - - - - - - - - - - - - - - - - - - */
/* EXTERNAL DATA- - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* PROTOTYPES - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/*
<HTML> 
<CENTER><HR>  Copyright (C) 1995, 1996 Wade Hampton (under GPL) </CENTER>
</HTML> 
*/
/************************************************************************/

/************************************************************************
<HTML>
<HR><H1> Procedure:  PrintHelp </H1><UL> 
<LI>     Purpose:    print help for this module
<LI>     Inputs:     none 
<LI>     Outputs:    stderr
</UL><H2>Notes: </H2><OL> 
<LI>Prints help info on standard output
</OL><H2>Requirements: </H2> <OL> 
<LI>(no number) - all programs should print help with -h, -H or -? options
</OL> 
</HTML> 
 ************************************************************************/
void PrintHelp()
{
  fprintf (stderr, PROG_NAME ":  Program to extract HTML from .c or .h\n");
  fprintf (stderr, "  Usage:  htmlextract -h  -- this help\n");
  fprintf (stderr, "          -iifile >ofile  -- specifiy input file\n");
  fprintf (stderr, "          <ifile >ofile   -- stdin for input\n");
  fprintf (stderr, "  Version:  %s\n", rcsid);
}
/************************************************************************
<HTML>
<HR><H1> Procedure:  PrintVersion </H1><UL> 
<LI>     Purpose:    print version for this module
<LI>     Inputs:     none 
<LI>     Outputs:    stderr
</UL> 
</UL><H2>Notes: </H2><OL> 
<LI>Prints version number for program
</OL><H2>Requirements: </H2> <OL> 
<LI>(no number) - all programs should print version with -V option
</OL> 

</HTML> 
 ************************************************************************/
void PrintVersion ()
{
  fprintf (stderr, PROG_NAME ":  version:  %s\n", rcsid);
}

/************************************************************************
<HTML>
<HR><H1> Procedure:  main </H1><UL> 
<LI>     Purpose:    module main procedure
<LI>     Inputs:     none 
<LI>     Outputs:    stdout, stderr

</UL><H2>Notes: </H2><OL> 
<LI>Usage:  htmlextract <ifile.[c,C,CC,h] >ifile.html
<LI> Extracts text bracketed by <HTML> and </HTML>
<LI> Puts <HTML> at start, </HTML> at end
<LI> Uses stdin for input, stdout for output, should be modified
     to accept a file name and cmd-line args (e.g., use getopt).

</OL><H2>Requirements: </H2> <OL> 
<LI> (1) Extract HTML from source code
</OL> 
</HTML> 
 ************************************************************************/

int main (argc, argv)
int argc;
char *argv[];
{

char caLineIn[1024];
int  iHtmlFlag  = FALSE;
char *pszInFile = "stdin";
int  iVerbose   = TRUE;
int  iLine      = 0;
int  iHLines    = 0;
time_t lTime;   
char   *pszAsciiTime;
int    ii;

  /* get current time into a string */
  time (&lTime);
  pszAsciiTime = ctime(&lTime);

  /* check cmd line args */
  if (argc > 1)
  {
    for (ii=1; ii<argc; ii++)
    {
      if (argv[ii][0] == '-')
      {
	switch (argv[ii][1])
	{
	case 'h':
	case 'H':
	case '?':
	  PrintHelp();
	  exit(0);
	  break;
	case 'V':
	  PrintVersion();
	  exit(0);
	  break;
	case 'i':
	  pszInFile = &argv[ii][2];
	  break;
	default:
	  PrintHelp();
	  exit(1);
	  break;
	}
      }
    }
  }

  /* try to open input file if not stdin */
  if (strcmp (pszInFile, "stdin") != 0)
  {
    INFILE = fopen (pszInFile, "r");
    if (INFILE == NULL)
    {
      fprintf (stderr, "Error opening %s, exiting\n", pszInFile);
      exit(1);
    }
  }
	  
  /* print HTML header info */
  printf ("<HTML>\n");
/*   printf ("<BODY>\n"); -- let user do this */
  printf ("<!Notes:  extracted from %s by %s>\n", pszInFile, PROG_NAME);
  printf ("<!        on date %s>\n", pszAsciiTime);
  iHtmlFlag = FALSE;
  while (fgets(caLineIn, sizeof(caLineIn)-2, INFILE) != NULL)
  {
    iLine++;
    /* if HTML off, check for start of HTML and if so, set flag */
    if (iHtmlFlag == FALSE)
    {
      if ((strncmp(HTML_START1, caLineIn, sizeof(HTML_START1)-1) == 0) ||
          (strncmp(HTML_START2, caLineIn, sizeof(HTML_START2)-1) == 0))
      {
        iHtmlFlag = TRUE;
        if (iVerbose == TRUE)
          fprintf (stderr, "%05d html ON\n", iLine);
      }
    } 
    /* if HTML on, check for end of HTML and if so, reset flag */
    else 
    {
      if ((strncmp(HTML_END1, caLineIn, sizeof(HTML_END1)-1) == 0) ||
          (strncmp(HTML_END2, caLineIn, sizeof(HTML_END2)-1) == 0))
      {
        iHtmlFlag = FALSE;
        if (iVerbose == TRUE)
          fprintf (stderr, "%05d html OFF\n", iLine);
      }
      if (iHtmlFlag == TRUE)
      {
        fputs (caLineIn, OUTFILE);
	iHLines++;
      }
    } 
  } 
/*  printf ("</BODY>\n"); -- let user do this */
  printf ("</HTML>\n");
  if (iVerbose == TRUE)
  {
    fprintf (stderr, "Total lines:  %d, HTML lines:  %d\n", iLine, iHLines);
  }

  exit(0);

}


/*
<HTML> 
</BODY>
</HTML> 
*/
